/*
 * Copyright (C) 2001 by Stefan Matthias Aust. All rights reserved.
 */
package sma.projects.game;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class GameClient extends JFrame {

    public GameClient(String host, int port) {
        super("GameClient");
        createUI();
        connect(host, port);
    }

    private DungeonUI dungeon;

    static long roundtripTime;

    private void createUI() {
        dungeon = new DungeonUI();
        getContentPane().add(dungeon, BorderLayout.CENTER);
        JLabel l = new JLabel("Copyright (c) 2001 Stefan Matthias Aust", JLabel.RIGHT);
        getContentPane().add(l, BorderLayout.SOUTH);
        pack();

        addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                try {
                    Log.log("got key, start roundtrip...");
                    roundtripTime = System.currentTimeMillis();
                    switch (e.getKeyCode()) {
                    case KeyEvent.VK_LEFT:
                        connection.sendCommand(0);
                        break;
                    case KeyEvent.VK_RIGHT:
                        connection.sendCommand(1);
                        break;
                    case KeyEvent.VK_UP:
                        connection.sendCommand(2);
                        break;
                    case KeyEvent.VK_DOWN:
                        connection.sendCommand(3);
                        break;
                    case KeyEvent.VK_SPACE:
                        dungeon.toggleMsg();
                    }
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        });

    }

    private Connection connection;

    private void connect(String host, int port) {
        try {
            connection = new Connection(host, port);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(this, "Cannot connect to server");
            System.exit(1);
        }
        connection.start();
    }

    private class Connection extends Thread {

        private Socket s;
        private DataInputStream in;
        private DataOutputStream out;

        Connection(String host, int port) throws IOException {
            this.s = new Socket(host, port);
            this.in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
            this.out = new DataOutputStream(s.getOutputStream());
        }

        void close() {
            try {
                in.close();
                out.close();
                s.close();
            } catch (IOException e) {
            }
        }

        void sendCommand(int cmd) throws IOException {
            out.writeInt(cmd);
        }

        public void run() {
            try {
                while (!Thread.interrupted()) {
                    dungeon.readTileCodes(in);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    static class DungeonUI extends JComponent {

        private static final int WIDTH = 15;
        private static final int HEIGHT = 15;
        private static final int TSIZE = 32;
        private static final Image i = Toolkit.getDefaultToolkit().getImage("tiles.gif");

        int[] tileCodes = new int[WIDTH * HEIGHT];

        DungeonUI() {
            setOpaque(true);
            setBackground(Color.black);

            i.getHeight(this);
        }

        // the usual partial loading of images causes a cascade of paint events
        // so we'll repaint just once after the image has been completely loaded
        public boolean imageUpdate(Image img, int infoflags,
                                   int x, int y, int width, int height) {
            if ((infoflags & ALLBITS) != 0)
                repaint();
            return (infoflags & (ALLBITS|ABORT)) == 0;
        }

        int X;

        public void paintComponent(Graphics g) {
            long t = System.currentTimeMillis();
            Rectangle r = g.getClipBounds();
            g.setColor(getBackground());
            g.fillRect(r.x, r.y, r.width, r.height);
            int y0 = Math.max(r.y / TSIZE, 0);
            int y1 = Math.min((r.y + r.height + TSIZE - 1) / TSIZE, HEIGHT);
            int x0 = Math.max(r.x / TSIZE, 0);
            int x1 = Math.min((r.x + r.width + TSIZE - 1) / TSIZE, WIDTH);
            Log.log("paint " + ++X + " " + r);
            for (int y = y0; y < y1; y++)
                for (int x = x0; x < x1; x++)
                    paintTile(g, x * TSIZE, y * TSIZE, tileCodes[y * WIDTH + x]);

            paintMessage(g);
            Log.log("paint time " + (System.currentTimeMillis() - t));
            Log.log("...roundtriptime " + (System.currentTimeMillis() - roundtripTime));
        }

        protected void paintTile(Graphics g, int x, int y, int tileCode) {
            int tc = tileCode;
            if (tc == 4) tc = 1;
            if (tc == 0)
                return;
            g.drawImage(i, x, y, x + TSIZE, y + TSIZE, tc * TSIZE, 0, tc * TSIZE + TSIZE, TSIZE, this);
            if (tileCode == 4)
                g.drawImage(i, x, y, x + TSIZE, y + TSIZE, tileCode * TSIZE, 0, tileCode * TSIZE + TSIZE, TSIZE, this);
        }

        private boolean msg;

        void toggleMsg() {
            msg = !msg;
            repaint();
        }

        protected void paintMessage(Graphics g) {
            if (!msg)
                return;
            int width = (int)(getWidth() * 0.7);
            int height = (int)(getHeight() * 0.5);
            int x = (getWidth() - width) / 2;
            int y = (getHeight() - height) / 2;
            g.setColor(new Color(204, 204, 216, 216));
            g.fillRect(x, y, width, height);
            g.setColor(Color.black);
            String msg = "This is just a demo";
            int o = (width - g.getFontMetrics().stringWidth(msg)) / 2;
            g.drawString(msg, x + o, y + 20);
        }

        public Dimension getPreferredSize() {
            return new Dimension(WIDTH * TSIZE, HEIGHT * TSIZE);
        }

        private void readTileCodes(DataInputStream in) throws IOException {
            for (int i = 0; i < tileCodes.length; i++)
                tileCodes[i] = in.readInt();
            Log.log("repaint");
            repaint();
        }
    }

    static public void main(String[] args) {
        int port = 29992;
        if (args.length > 1)
            port = Integer.parseInt(args[1]);
        String host = "localhost";
        if (args.length > 0)
            host = args[0];
        new GameClient(host, port).show();
    }
}