/*
 * Copyright (C) 2001 by Stefan Matthias Aust. All rights reserved.
 */
package sma.projects.game;

import java.io.*;
import java.net.*;
import java.util.*;

public class GameServer {

    private List clients = new LinkedList();

    /**
     * Runs the central server loop that waits for client connections.
     */
    public void run(int port) throws IOException {
        createDungeon();
        new GameLoop().start();

        ServerSocket ss = new ServerSocket(port);
        Log.log("server is running");
        while (!Thread.interrupted())
            newConnection(ss.accept());
        Log.log("server has stopped");
    }

    /**
     * Creates and spawns a new client handler for the specified connection.
     */
    protected void newConnection(Socket s) throws IOException {
        Log.log("new connection via " + s);
        ClientHandler h = new ClientHandler(s);
        synchronized (clients) {
            clients.add(h);
        }
        h.start();
        Log.log("client added");
    }

    /**
     * Removes the specified client handler from the server.
     */
    protected void removeClient(ClientHandler h) {
        synchronized (clients) {
            clients.remove(h);
        }
        Log.log("client removed");
    }

    /**
     * Returns the list of active clients in a thread-safe manner.
     */
    protected List getClients() {
        synchronized (clients) {
            return new LinkedList(clients);
        }
    }

    /**
     * A <code>ClientHandler</code> maintains one client/server connection.
     */
    protected class ClientHandler extends Thread {

        private Socket s;
        private DataInputStream in;
        private DataOutputStream out;

        ClientHandler(Socket s) throws IOException {
            this.s = s;
            this.in = new DataInputStream(s.getInputStream());
            this.out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
        }

        public void run() {
            sendView();
            try {
                while (!interrupted()) {
                    setCommand(in.readInt());
                }
            } catch (IOException e) {
                // client breakdown
            }
            try {
                in.close();
                out.close();
                s.close();
            } catch (IOException e) {
            }
            removeClient(this);
        }

        // ----------------------------------------------------------------------

        private Hero hero = createHero();

        private void setCommand(int cmd) {
            Log.log("got client command " + cmd);

            switch (cmd) {
            case 0:
                hero.goWest();
                break;
            case 1:
                hero.goEast();
                break;
            case 2:
                hero.goNorth();
                break;
            case 3:
                hero.goSouth();
                break;
            }
            needBroadcast = true;
        }

        private void sendView() {
            try {
                hero.sendTileCodes(out);
            } catch (IOException e) {
                removeClient(this);
            }
        }
    }

    // ----------------------------------------------------------------------

    private Dungeon dungeon;

    void createDungeon() {
        Random r = new Random();
        dungeon = new Dungeon(43, 43);
        dungeon.createLevel(r);
    }

    Hero createHero() {
        Random r = new Random();
        Hero h = new Hero(dungeon);
        int x, y;
        do {
            x = r.nextInt(dungeon.getWidth() - 2) + 1;
            y = r.nextInt(dungeon.getHeight() - 2) + 1;
        } while (!h.canEnter(x, y));
        h.go(x, y);

        return h;
    }

    boolean needBroadcast;

    synchronized void broadcastViews() {
        needBroadcast = false;
        Iterator i = getClients().iterator();
        while (i.hasNext()) {
            ((ClientHandler)i.next()).sendView();
        }
    }

    class GameLoop extends Thread {

        public void run() {
            while (!Thread.interrupted()) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    break;
                }
                if (needBroadcast)
                    broadcastViews();
            }
        }
    }

    // ----------------------------------------------------------------------

    static public void main(String[] args) {
        int port = 29992;
        if (args.length > 0)
            port = Integer.parseInt(args[0]);
        try {
            new GameServer().run(port);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}